#!/bin/sh
# MagicZaurus January 2009 for Puppy Linux GPL.
# Based on script from SLAX.
# Enhanced by Ecomoney August 2009
# Customized by Ph. Lelédy

# Updated by jamesbond 2011 - add control from GUI
# the base directory is now fixed at /root/tftpboot (this can be moved elsewhere by symlink)
# assumes that ip, sed and dnsmasq is somewhere on the path
# version 2 - combines normal and debug mode

# get_ip get and parse ip address - output is interface name and its ip address
# only show interfaces with active ip address, and excludes the local interface
function get_ip() {
	ip address | sed -ne '
/^[0-9]:/ {
	# extract the interface name
	s/^[0-9]: *\([a-z0-9]*\):.*/\1/
	# keep it for later
	h
}

/inet/ { 
	# extract the ip address
	s/^ *inet \([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*\).*/\1/
	# add this to any pattern we found before
	H
	# Now get the whole thing
	g
	# and remove the newlines in between - change that to a space
	s/\n/ /
	# and print it
	/^lo/ !p
	#p
}
'
}

# run dnsmasq - the key component in this whole script
function run_dnsmasq() {
	mkdir -p /var/lib/misc/
	dnsmasq  \
	 --port=0 \
	 --dhcp-vendorclass=set:PXE,PXEClient \
	 --tag-if=set:notPXE,tag:!PXE \
	 --dhcp-ignore=tag:notPXE \
	 --dhcp-range=tag:PXE,$IP_BASE.110,$IP_BASE.200 \
	 --no-ping \
	 --dhcp-boot=pxelinux.0 \
	 --dhcp-option-force=tag:PXE,208,F1:00:74:7E \
	 --dhcp-option-force=tag:PXE,209,"default" \
	 --dhcp-option-force=tag:PXE,210,"pxelinux.cfg/" \
	 --dhcp-option-force=tag:PXE,211,30 \
	 --enable-tftp \
	 --tftp-root=$TFTP_ROOT \
	 $* 2> /tmp/dnsmasq-debug.log &
	DNSMASQ_PID=$!	
}

# run dnsmasq with debug flag set, capture output in a window
function run_dnsmasq_debug() {
	run_dnsmasq -d
	Xdialog --title "Netboot server" --backtitle "Running server in debug mode. Click cancel to quit and stop server." --no-ok --tailbox /tmp/dnsmasq-debug.log 20 100
	kill $DNSMASQ_PID
	return 0
}

# run dnsmasq normally
function run_dnsmasq_normal() {
	run_dnsmasq
	Xdialog --title "Netboot Server" --infobox "Server started successfully. Click on menu again to stop." 0 0 5000
	return 0
}

# === main ===

# check if dnsmasq is already running, if it is, ask whether we want to terminate
if ps -C dnsmasq > /dev/null; then
	if Xdialog --title "Netboot Server" --yesno "Server is already running, do you want to stop it?" 0 0; then
		killall dnsmasq 2> /dev/null
	fi
	exit 2;
fi

# get ip address info
if [ $(get_ip|wc -l) -gt 1 ]; then
	# if more than 1 active interfaces, let user choose which one to use
	CHOICES=$(get_ip | while read ifname ipaddress; do
		echo "$ipaddress $ifname off"
	done)
	ACTIVE_IP=$(Xdialog --title "Netboot Server" --no-tags --radiolist "Choose the interfaces you want to use" 0 0 5 $CHOICES 2>&1)
else
	ACTIVE_IP=$(get_ip)
fi
if [ -z "$ACTIVE_IP" ]; then exit 1; fi

# echo $ACTIVE_IP
PXE_IP=$(echo $ACTIVE_IP | sed 's/^.* //' )
IP_BASE=$(echo $PXE_IP | cut -d "." -f 1-3)
TFTP_ROOT="/root/tftpboot"

# sanity check - check for root directory
if [ ! -e $TFTP_ROOT ]; then
	Xdialog --title "Netboot Server" --infobox "$TFTP_ROOT directory does not exist, create one first!" 0 0 5000
	exit 1;
fi

# how to run it
MODE=$(Xdialog --title "Netboot Server" --no-tags --menubox "How do you want to start the server?" 0 0 3 "normal" "Normal" "debug" "Debug mode" 2>&1)
case "$MODE" in 
	normal) run_dnsmasq_normal ;;
	debug) run_dnsmasq_debug ;;
	*) exit 1 ;;
esac
